# Form implementation generated from reading ui file 'src/magboltz_gui/ui/main.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1301, 912)
        self.centralwidget = QtWidgets.QWidget(parent=MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.mainTab = QtWidgets.QTabWidget(parent=self.centralwidget)
        self.mainTab.setObjectName("mainTab")
        self.tabConfiguration = QtWidgets.QWidget()
        self.tabConfiguration.setObjectName("tabConfiguration")
        self.gridLayout_31 = QtWidgets.QGridLayout(self.tabConfiguration)
        self.gridLayout_31.setObjectName("gridLayout_31")
        self.groupSimParameters = QtWidgets.QGroupBox(parent=self.tabConfiguration)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupSimParameters.sizePolicy().hasHeightForWidth())
        self.groupSimParameters.setSizePolicy(sizePolicy)
        self.groupSimParameters.setObjectName("groupSimParameters")
        self.gridLayout = QtWidgets.QGridLayout(self.groupSimParameters)
        self.gridLayout.setObjectName("gridLayout")
        self.label_4 = QtWidgets.QLabel(parent=self.groupSimParameters)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 1, 2, 1, 1)
        self.spinFinalEnergy = QtWidgets.QDoubleSpinBox(parent=self.groupSimParameters)
        self.spinFinalEnergy.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.spinFinalEnergy.setObjectName("spinFinalEnergy")
        self.gridLayout.addWidget(self.spinFinalEnergy, 1, 1, 1, 1)
        self.label = QtWidgets.QLabel(parent=self.groupSimParameters)
        self.label.setMinimumSize(QtCore.QSize(200, 0))
        self.label.setToolTip("")
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.label_17 = QtWidgets.QLabel(parent=self.groupSimParameters)
        self.label_17.setObjectName("label_17")
        self.gridLayout.addWidget(self.label_17, 4, 0, 1, 1)
        self.spinRealInteractions = QtWidgets.QSpinBox(parent=self.groupSimParameters)
        self.spinRealInteractions.setMinimumSize(QtCore.QSize(100, 0))
        self.spinRealInteractions.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.spinRealInteractions.setObjectName("spinRealInteractions")
        self.gridLayout.addWidget(self.spinRealInteractions, 0, 1, 1, 1)
        self.checkPenning = QtWidgets.QCheckBox(parent=self.groupSimParameters)
        self.checkPenning.setMinimumSize(QtCore.QSize(0, 25))
        self.checkPenning.setText("")
        self.checkPenning.setObjectName("checkPenning")
        self.gridLayout.addWidget(self.checkPenning, 2, 1, 1, 1)
        self.checkFinalEnergyAuto = QtWidgets.QCheckBox(parent=self.groupSimParameters)
        self.checkFinalEnergyAuto.setObjectName("checkFinalEnergyAuto")
        self.gridLayout.addWidget(self.checkFinalEnergyAuto, 1, 4, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 4, 1, 1)
        self.label_3 = QtWidgets.QLabel(parent=self.groupSimParameters)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 0, 2, 1, 1)
        self.label_16 = QtWidgets.QLabel(parent=self.groupSimParameters)
        self.label_16.setObjectName("label_16")
        self.gridLayout.addWidget(self.label_16, 2, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(parent=self.groupSimParameters)
        self.label_2.setMinimumSize(QtCore.QSize(0, 0))
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.checkThermal = QtWidgets.QCheckBox(parent=self.groupSimParameters)
        self.checkThermal.setMinimumSize(QtCore.QSize(0, 25))
        self.checkThermal.setText("")
        self.checkThermal.setObjectName("checkThermal")
        self.gridLayout.addWidget(self.checkThermal, 4, 1, 1, 1)
        self.gridLayout_31.addWidget(self.groupSimParameters, 0, 0, 1, 1)
        self.groupGasList = QtWidgets.QGroupBox(parent=self.tabConfiguration)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupGasList.sizePolicy().hasHeightForWidth())
        self.groupGasList.setSizePolicy(sizePolicy)
        self.groupGasList.setObjectName("groupGasList")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupGasList)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_12 = QtWidgets.QLabel(parent=self.groupGasList)
        self.label_12.setObjectName("label_12")
        self.gridLayout_2.addWidget(self.label_12, 0, 2, 1, 1)
        self.label_6 = QtWidgets.QLabel(parent=self.groupGasList)
        self.label_6.setObjectName("label_6")
        self.gridLayout_2.addWidget(self.label_6, 2, 2, 1, 1)
        self.spinGasTemperature = QtWidgets.QDoubleSpinBox(parent=self.groupGasList)
        self.spinGasTemperature.setMinimumSize(QtCore.QSize(100, 0))
        self.spinGasTemperature.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.spinGasTemperature.setDecimals(1)
        self.spinGasTemperature.setMaximum(500.0)
        self.spinGasTemperature.setObjectName("spinGasTemperature")
        self.gridLayout_2.addWidget(self.spinGasTemperature, 0, 1, 1, 1)
        self.spinGasPressure = QtWidgets.QDoubleSpinBox(parent=self.groupGasList)
        self.spinGasPressure.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.spinGasPressure.setDecimals(0)
        self.spinGasPressure.setMaximum(10000.0)
        self.spinGasPressure.setSingleStep(10.0)
        self.spinGasPressure.setObjectName("spinGasPressure")
        self.gridLayout_2.addWidget(self.spinGasPressure, 1, 1, 1, 1)
        self.spinMagneticField = QtWidgets.QDoubleSpinBox(parent=self.groupGasList)
        self.spinMagneticField.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.spinMagneticField.setDecimals(3)
        self.spinMagneticField.setMaximum(10.0)
        self.spinMagneticField.setObjectName("spinMagneticField")
        self.gridLayout_2.addWidget(self.spinMagneticField, 3, 1, 1, 1)
        self.label_8 = QtWidgets.QLabel(parent=self.groupGasList)
        self.label_8.setObjectName("label_8")
        self.gridLayout_2.addWidget(self.label_8, 3, 2, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem1, 0, 4, 1, 1)
        self.label_9 = QtWidgets.QLabel(parent=self.groupGasList)
        self.label_9.setMinimumSize(QtCore.QSize(200, 0))
        self.label_9.setObjectName("label_9")
        self.gridLayout_2.addWidget(self.label_9, 4, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(parent=self.groupGasList)
        self.label_7.setMinimumSize(QtCore.QSize(200, 0))
        self.label_7.setObjectName("label_7")
        self.gridLayout_2.addWidget(self.label_7, 3, 0, 1, 1)
        self.label_11 = QtWidgets.QLabel(parent=self.groupGasList)
        self.label_11.setMinimumSize(QtCore.QSize(200, 0))
        self.label_11.setObjectName("label_11")
        self.gridLayout_2.addWidget(self.label_11, 0, 0, 1, 1)
        self.spinAngle = QtWidgets.QDoubleSpinBox(parent=self.groupGasList)
        self.spinAngle.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.spinAngle.setMaximum(359.99)
        self.spinAngle.setObjectName("spinAngle")
        self.gridLayout_2.addWidget(self.spinAngle, 4, 1, 1, 1)
        self.label_14 = QtWidgets.QLabel(parent=self.groupGasList)
        self.label_14.setObjectName("label_14")
        self.gridLayout_2.addWidget(self.label_14, 1, 2, 1, 1)
        self.label_5 = QtWidgets.QLabel(parent=self.groupGasList)
        self.label_5.setMinimumSize(QtCore.QSize(200, 0))
        self.label_5.setObjectName("label_5")
        self.gridLayout_2.addWidget(self.label_5, 2, 0, 1, 1)
        self.label_13 = QtWidgets.QLabel(parent=self.groupGasList)
        self.label_13.setObjectName("label_13")
        self.gridLayout_2.addWidget(self.label_13, 1, 0, 1, 1)
        self.spinElectricField = QtWidgets.QDoubleSpinBox(parent=self.groupGasList)
        self.spinElectricField.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.spinElectricField.setDecimals(0)
        self.spinElectricField.setMaximum(100000.0)
        self.spinElectricField.setSingleStep(100.0)
        self.spinElectricField.setObjectName("spinElectricField")
        self.gridLayout_2.addWidget(self.spinElectricField, 2, 1, 1, 1)
        self.label_10 = QtWidgets.QLabel(parent=self.groupGasList)
        self.label_10.setObjectName("label_10")
        self.gridLayout_2.addWidget(self.label_10, 4, 2, 1, 1)
        self.gridLayout_31.addWidget(self.groupGasList, 0, 1, 1, 1)
        self.groupGases = QtWidgets.QGroupBox(parent=self.tabConfiguration)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupGases.sizePolicy().hasHeightForWidth())
        self.groupGases.setSizePolicy(sizePolicy)
        self.groupGases.setObjectName("groupGases")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.groupGases)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.splitterGases = QtWidgets.QSplitter(parent=self.groupGases)
        self.splitterGases.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.splitterGases.setObjectName("splitterGases")
        self.splitterGasesLeft = QtWidgets.QWidget(parent=self.splitterGases)
        self.splitterGasesLeft.setObjectName("splitterGasesLeft")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.splitterGasesLeft)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.widget = QtWidgets.QWidget(parent=self.splitterGasesLeft)
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btnGasAdd = QtWidgets.QToolButton(parent=self.widget)
        icon = QtGui.QIcon.fromTheme("list-add")
        self.btnGasAdd.setIcon(icon)
        self.btnGasAdd.setObjectName("btnGasAdd")
        self.horizontalLayout.addWidget(self.btnGasAdd)
        self.btnGasRemove = QtWidgets.QToolButton(parent=self.widget)
        icon = QtGui.QIcon.fromTheme("list-remove")
        self.btnGasRemove.setIcon(icon)
        self.btnGasRemove.setObjectName("btnGasRemove")
        self.horizontalLayout.addWidget(self.btnGasRemove)
        self.btnGasNormalize = QtWidgets.QToolButton(parent=self.widget)
        icon = QtGui.QIcon.fromTheme("accessories-calculator")
        self.btnGasNormalize.setIcon(icon)
        self.btnGasNormalize.setObjectName("btnGasNormalize")
        self.horizontalLayout.addWidget(self.btnGasNormalize)
        self.lblNeedNormalize = QtWidgets.QLabel(parent=self.widget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(192, 28, 40))
        brush.setStyle(QtCore.Qt.BrushStyle.SolidPattern)
        palette.setBrush(QtGui.QPalette.ColorGroup.Active, QtGui.QPalette.ColorRole.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.BrushStyle.SolidPattern)
        palette.setBrush(QtGui.QPalette.ColorGroup.Inactive, QtGui.QPalette.ColorRole.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(190, 190, 190))
        brush.setStyle(QtCore.Qt.BrushStyle.SolidPattern)
        palette.setBrush(QtGui.QPalette.ColorGroup.Disabled, QtGui.QPalette.ColorRole.WindowText, brush)
        self.lblNeedNormalize.setPalette(palette)
        font = QtGui.QFont()
        font.setBold(True)
        self.lblNeedNormalize.setFont(font)
        self.lblNeedNormalize.setObjectName("lblNeedNormalize")
        self.horizontalLayout.addWidget(self.lblNeedNormalize)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.btnGasUp = QtWidgets.QToolButton(parent=self.widget)
        icon = QtGui.QIcon.fromTheme("go-up")
        self.btnGasUp.setIcon(icon)
        self.btnGasUp.setObjectName("btnGasUp")
        self.horizontalLayout.addWidget(self.btnGasUp)
        self.btnGasDown = QtWidgets.QToolButton(parent=self.widget)
        icon = QtGui.QIcon.fromTheme("go-down")
        self.btnGasDown.setIcon(icon)
        self.btnGasDown.setObjectName("btnGasDown")
        self.horizontalLayout.addWidget(self.btnGasDown)
        self.verticalLayout_2.addWidget(self.widget)
        self.gasListTable = QtWidgets.QTableWidget(parent=self.splitterGasesLeft)
        self.gasListTable.setColumnCount(3)
        self.gasListTable.setObjectName("gasListTable")
        self.gasListTable.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.gasListTable.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        item.setTextAlignment(QtCore.Qt.AlignmentFlag.AlignLeading|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.gasListTable.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        item.setTextAlignment(QtCore.Qt.AlignmentFlag.AlignLeading|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.gasListTable.setHorizontalHeaderItem(2, item)
        self.verticalLayout_2.addWidget(self.gasListTable)
        self.splitterGasRight = QtWidgets.QWidget(parent=self.splitterGases)
        self.splitterGasRight.setObjectName("splitterGasRight")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.splitterGasRight)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.widget_3 = QtWidgets.QWidget(parent=self.splitterGasRight)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget_3.sizePolicy().hasHeightForWidth())
        self.widget_3.setSizePolicy(sizePolicy)
        self.widget_3.setObjectName("widget_3")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.widget_3)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_18 = QtWidgets.QLabel(parent=self.widget_3)
        self.label_18.setObjectName("label_18")
        self.horizontalLayout_4.addWidget(self.label_18)
        self.cmbLabelFormat = QtWidgets.QComboBox(parent=self.widget_3)
        self.cmbLabelFormat.setObjectName("cmbLabelFormat")
        self.cmbLabelFormat.addItem("")
        self.cmbLabelFormat.addItem("")
        self.cmbLabelFormat.addItem("")
        self.cmbLabelFormat.addItem("")
        self.horizontalLayout_4.addWidget(self.cmbLabelFormat)
        self.label_19 = QtWidgets.QLabel(parent=self.widget_3)
        self.label_19.setObjectName("label_19")
        self.horizontalLayout_4.addWidget(self.label_19)
        self.cmbColorMap = QtWidgets.QComboBox(parent=self.widget_3)
        self.cmbColorMap.setObjectName("cmbColorMap")
        self.horizontalLayout_4.addWidget(self.cmbColorMap)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem3)
        self.btnGraphSave = QtWidgets.QToolButton(parent=self.widget_3)
        icon = QtGui.QIcon.fromTheme("document-save-as")
        self.btnGraphSave.setIcon(icon)
        self.btnGraphSave.setIconSize(QtCore.QSize(16, 16))
        self.btnGraphSave.setToolButtonStyle(QtCore.Qt.ToolButtonStyle.ToolButtonIconOnly)
        self.btnGraphSave.setObjectName("btnGraphSave")
        self.horizontalLayout_4.addWidget(self.btnGraphSave)
        self.verticalLayout_5.addWidget(self.widget_3)
        self.pieContainer = QtWidgets.QFrame(parent=self.splitterGasRight)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pieContainer.sizePolicy().hasHeightForWidth())
        self.pieContainer.setSizePolicy(sizePolicy)
        self.pieContainer.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.pieContainer.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.pieContainer.setObjectName("pieContainer")
        self.verticalLayout_5.addWidget(self.pieContainer)
        self.horizontalLayout_3.addWidget(self.splitterGases)
        self.gridLayout_31.addWidget(self.groupGases, 1, 0, 1, 2)
        self.mainTab.addTab(self.tabConfiguration, "")
        self.tabExecution = QtWidgets.QWidget()
        self.tabExecution.setObjectName("tabExecution")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tabExecution)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.widget_2 = QtWidgets.QWidget(parent=self.tabExecution)
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_15 = QtWidgets.QLabel(parent=self.widget_2)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_2.addWidget(self.label_15)
        self.commandLine = QtWidgets.QLineEdit(parent=self.widget_2)
        self.commandLine.setReadOnly(True)
        self.commandLine.setObjectName("commandLine")
        self.horizontalLayout_2.addWidget(self.commandLine)
        self.btnCmdCopyToClipbord = QtWidgets.QToolButton(parent=self.widget_2)
        icon = QtGui.QIcon.fromTheme("edit-copy")
        self.btnCmdCopyToClipbord.setIcon(icon)
        self.btnCmdCopyToClipbord.setObjectName("btnCmdCopyToClipbord")
        self.horizontalLayout_2.addWidget(self.btnCmdCopyToClipbord)
        self.btnResultSave = QtWidgets.QToolButton(parent=self.widget_2)
        icon = QtGui.QIcon.fromTheme("document-save-as")
        self.btnResultSave.setIcon(icon)
        self.btnResultSave.setObjectName("btnResultSave")
        self.horizontalLayout_2.addWidget(self.btnResultSave)
        self.btnResultExport = QtWidgets.QToolButton(parent=self.widget_2)
        icon = QtGui.QIcon.fromTheme("x-office-spreadsheet")
        self.btnResultExport.setIcon(icon)
        self.btnResultExport.setObjectName("btnResultExport")
        self.horizontalLayout_2.addWidget(self.btnResultExport)
        self.verticalLayout_4.addWidget(self.widget_2)
        self.consoleOutput = QtWidgets.QTextEdit(parent=self.tabExecution)
        font = QtGui.QFont()
        font.setFamily("Monospace")
        self.consoleOutput.setFont(font)
        self.consoleOutput.setReadOnly(True)
        self.consoleOutput.setObjectName("consoleOutput")
        self.verticalLayout_4.addWidget(self.consoleOutput)
        self.mainTab.addTab(self.tabExecution, "")
        self.gridLayout_3.addWidget(self.mainTab, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.mainMenuBar = QtWidgets.QMenuBar(parent=MainWindow)
        self.mainMenuBar.setGeometry(QtCore.QRect(0, 0, 1301, 23))
        self.mainMenuBar.setObjectName("mainMenuBar")
        self.menu_File = QtWidgets.QMenu(parent=self.mainMenuBar)
        self.menu_File.setObjectName("menu_File")
        self.menu_Edit = QtWidgets.QMenu(parent=self.mainMenuBar)
        self.menu_Edit.setObjectName("menu_Edit")
        self.menu_Help = QtWidgets.QMenu(parent=self.mainMenuBar)
        self.menu_Help.setSeparatorsCollapsible(False)
        self.menu_Help.setObjectName("menu_Help")
        self.menuRun = QtWidgets.QMenu(parent=self.mainMenuBar)
        self.menuRun.setObjectName("menuRun")
        MainWindow.setMenuBar(self.mainMenuBar)
        self.mainToolBar = QtWidgets.QToolBar(parent=MainWindow)
        self.mainToolBar.setObjectName("mainToolBar")
        MainWindow.addToolBar(QtCore.Qt.ToolBarArea.TopToolBarArea, self.mainToolBar)
        self.actionNew = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("document-new")
        self.actionNew.setIcon(icon)
        self.actionNew.setObjectName("actionNew")
        self.actionOpen = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("document-open")
        self.actionOpen.setIcon(icon)
        self.actionOpen.setObjectName("actionOpen")
        self.actionQuit = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("application-exit")
        self.actionQuit.setIcon(icon)
        self.actionQuit.setObjectName("actionQuit")
        self.actionSave = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("document-save")
        self.actionSave.setIcon(icon)
        self.actionSave.setObjectName("actionSave")
        self.actionClose = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("edit-delete")
        self.actionClose.setIcon(icon)
        self.actionClose.setObjectName("actionClose")
        self.actionSaveAs = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("document-save-as")
        self.actionSaveAs.setIcon(icon)
        self.actionSaveAs.setObjectName("actionSaveAs")
        self.actionRevert = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("document-revert")
        self.actionRevert.setIcon(icon)
        self.actionRevert.setObjectName("actionRevert")
        self.actionRun = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("system-run")
        self.actionRun.setIcon(icon)
        self.actionRun.setObjectName("actionRun")
        self.actionGasAdd = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("list-add")
        self.actionGasAdd.setIcon(icon)
        self.actionGasAdd.setObjectName("actionGasAdd")
        self.actionGasRemove = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("list-remove")
        self.actionGasRemove.setIcon(icon)
        self.actionGasRemove.setObjectName("actionGasRemove")
        self.actionGasNormalize = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("accessories-calculator")
        self.actionGasNormalize.setIcon(icon)
        self.actionGasNormalize.setObjectName("actionGasNormalize")
        self.actionResultExport = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("x-office-spreadsheet")
        self.actionResultExport.setIcon(icon)
        self.actionResultExport.setObjectName("actionResultExport")
        self.actionCmdCopyToClipboard = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("edit-copy")
        self.actionCmdCopyToClipboard.setIcon(icon)
        self.actionCmdCopyToClipboard.setObjectName("actionCmdCopyToClipboard")
        self.actionResultSave = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("document-save-as")
        self.actionResultSave.setIcon(icon)
        self.actionResultSave.setObjectName("actionResultSave")
        self.actionGraphSave = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("document-save-as")
        self.actionGraphSave.setIcon(icon)
        self.actionGraphSave.setObjectName("actionGraphSave")
        self.actionGasMoveUp = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("go-up")
        self.actionGasMoveUp.setIcon(icon)
        self.actionGasMoveUp.setObjectName("actionGasMoveUp")
        self.actionGasMoveDown = QtGui.QAction(parent=MainWindow)
        icon = QtGui.QIcon.fromTheme("go-down")
        self.actionGasMoveDown.setIcon(icon)
        self.actionGasMoveDown.setObjectName("actionGasMoveDown")
        self.menu_File.addAction(self.actionNew)
        self.menu_File.addAction(self.actionOpen)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.actionSave)
        self.menu_File.addAction(self.actionSaveAs)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.actionRevert)
        self.menu_File.addAction(self.actionClose)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.actionQuit)
        self.menu_Edit.addSeparator()
        self.menuRun.addAction(self.actionRun)
        self.menuRun.addSeparator()
        self.menuRun.addAction(self.actionResultSave)
        self.menuRun.addAction(self.actionResultExport)
        self.mainMenuBar.addAction(self.menu_File.menuAction())
        self.mainMenuBar.addAction(self.menu_Edit.menuAction())
        self.mainMenuBar.addAction(self.menuRun.menuAction())
        self.mainMenuBar.addAction(self.menu_Help.menuAction())
        self.mainToolBar.addAction(self.actionNew)
        self.mainToolBar.addAction(self.actionOpen)
        self.mainToolBar.addSeparator()
        self.mainToolBar.addAction(self.actionSave)
        self.mainToolBar.addAction(self.actionSaveAs)
        self.mainToolBar.addSeparator()
        self.mainToolBar.addAction(self.actionRevert)
        self.mainToolBar.addSeparator()
        self.mainToolBar.addAction(self.actionRun)

        self.retranslateUi(MainWindow)
        self.mainTab.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Magboltz GUI"))
        self.groupSimParameters.setTitle(_translate("MainWindow", "SImulation parameters"))
        self.label_4.setText(_translate("MainWindow", "eV"))
        self.spinFinalEnergy.setToolTip(_translate("MainWindow", "EFINAL: upper limit of the electron energy in electron Volts, if EFINAL = 0.0, program automatically calculates upper integration energy limit"))
        self.label.setText(_translate("MainWindow", "Real interactions"))
        self.label_17.setText(_translate("MainWindow", "Enable thermal motion"))
        self.spinRealInteractions.setToolTip(_translate("MainWindow", " NMAX: Number of real collisions, use a value between 2 and 5 for inelastic gas to obtain 1% accuracy, use a value above 10 for better than 0.5 % accuracy and a value of at least 10 for pure elastic gases like Argon"))
        self.checkPenning.setToolTip(_translate("MainWindow", "IPEN: include Penning effect"))
        self.checkFinalEnergyAuto.setText(_translate("MainWindow", "Auto"))
        self.label_3.setText(_translate("MainWindow", "× 10⁷"))
        self.label_16.setText(_translate("MainWindow", "Enable Penning effect"))
        self.label_2.setText(_translate("MainWindow", "Final energy"))
        self.checkThermal.setToolTip(_translate("MainWindow", "ITHRM: gas motion taken to be at input temperature"))
        self.groupGasList.setTitle(_translate("MainWindow", "Conditions"))
        self.label_12.setText(_translate("MainWindow", "℃"))
        self.label_6.setText(_translate("MainWindow", "V/cm"))
        self.spinGasTemperature.setToolTip(_translate("MainWindow", "TEMP: temperature of gas in centigrade"))
        self.spinGasPressure.setToolTip(_translate("MainWindow", "TORR: pressure of gas in Torr"))
        self.spinMagneticField.setToolTip(_translate("MainWindow", "BMAG: magnitude of the magnetic field in kilogauss"))
        self.label_8.setText(_translate("MainWindow", "kG"))
        self.label_9.setText(_translate("MainWindow", "Angle"))
        self.label_7.setText(_translate("MainWindow", "Magnetic field"))
        self.label_11.setText(_translate("MainWindow", "Gas temperature"))
        self.spinAngle.setToolTip(_translate("MainWindow", "BTHETA: angle between the electric and magnetic fields in degrees"))
        self.label_14.setText(_translate("MainWindow", "Torr"))
        self.label_5.setText(_translate("MainWindow", "Electric field"))
        self.label_13.setText(_translate("MainWindow", "Gas pressure"))
        self.spinElectricField.setToolTip(_translate("MainWindow", "EFIELD: electric field in Volt/cm"))
        self.label_10.setText(_translate("MainWindow", "°"))
        self.groupGases.setTitle(_translate("MainWindow", "Gas components"))
        self.btnGasAdd.setText(_translate("MainWindow", "..."))
        self.btnGasRemove.setText(_translate("MainWindow", "..."))
        self.btnGasNormalize.setToolTip(_translate("MainWindow", "Normalize gas fractions"))
        self.btnGasNormalize.setText(_translate("MainWindow", "..."))
        self.lblNeedNormalize.setText(_translate("MainWindow", "gas fractions needs to be normalized"))
        self.btnGasUp.setText(_translate("MainWindow", "..."))
        self.btnGasDown.setText(_translate("MainWindow", "..."))
        item = self.gasListTable.horizontalHeaderItem(0)
        item.setText(_translate("MainWindow", "Gas ID"))
        item = self.gasListTable.horizontalHeaderItem(1)
        item.setText(_translate("MainWindow", "Gas name"))
        item = self.gasListTable.horizontalHeaderItem(2)
        item.setText(_translate("MainWindow", "Gas fraction"))
        self.label_18.setText(_translate("MainWindow", "Label"))
        self.cmbLabelFormat.setItemText(0, _translate("MainWindow", "Formula"))
        self.cmbLabelFormat.setItemText(1, _translate("MainWindow", "Name"))
        self.cmbLabelFormat.setItemText(2, _translate("MainWindow", "Name (formula)"))
        self.cmbLabelFormat.setItemText(3, _translate("MainWindow", "None"))
        self.label_19.setText(_translate("MainWindow", "Colors"))
        self.btnGraphSave.setText(_translate("MainWindow", "..."))
        self.mainTab.setTabText(self.mainTab.indexOf(self.tabConfiguration), _translate("MainWindow", "Configuration"))
        self.label_15.setText(_translate("MainWindow", "Command line"))
        self.commandLine.setPlaceholderText(_translate("MainWindow", "(save the current file)"))
        self.btnCmdCopyToClipbord.setText(_translate("MainWindow", "..."))
        self.btnResultSave.setText(_translate("MainWindow", "..."))
        self.btnResultExport.setText(_translate("MainWindow", "..."))
        self.mainTab.setTabText(self.mainTab.indexOf(self.tabExecution), _translate("MainWindow", "Execution"))
        self.menu_File.setTitle(_translate("MainWindow", "&File"))
        self.menu_Edit.setTitle(_translate("MainWindow", "&Edit"))
        self.menu_Help.setTitle(_translate("MainWindow", "&Help"))
        self.menuRun.setTitle(_translate("MainWindow", "Run"))
        self.mainToolBar.setWindowTitle(_translate("MainWindow", "toolBar"))
        self.actionNew.setText(_translate("MainWindow", "&New"))
        self.actionOpen.setText(_translate("MainWindow", "&Open"))
        self.actionQuit.setText(_translate("MainWindow", "&Exit"))
        self.actionSave.setText(_translate("MainWindow", "&Save"))
        self.actionClose.setText(_translate("MainWindow", "Close"))
        self.actionSaveAs.setText(_translate("MainWindow", "Save As"))
        self.actionRevert.setText(_translate("MainWindow", "Revert"))
        self.actionRun.setText(_translate("MainWindow", "Run"))
        self.actionGasAdd.setText(_translate("MainWindow", "Add"))
        self.actionGasRemove.setText(_translate("MainWindow", "Remove"))
        self.actionGasNormalize.setText(_translate("MainWindow", "Normalize"))
        self.actionGasNormalize.setToolTip(_translate("MainWindow", "Normalize"))
        self.actionResultExport.setText(_translate("MainWindow", "Export"))
        self.actionCmdCopyToClipboard.setText(_translate("MainWindow", "Cooy to clipboard"))
        self.actionCmdCopyToClipboard.setShortcut(_translate("MainWindow", "Ctrl+C"))
        self.actionResultSave.setText(_translate("MainWindow", "Save"))
        self.actionGraphSave.setText(_translate("MainWindow", "Save graph"))
        self.actionGasMoveUp.setText(_translate("MainWindow", "Move up"))
        self.actionGasMoveDown.setText(_translate("MainWindow", "Move down"))



from pathlib import Path
import subprocess
import sys


def compile_ui(ui_dir: str, out_dir: str) -> int:
    src = Path(ui_dir)
    out = Path(out_dir)
    here = Path(__file__).resolve().parent

    out.mkdir(parents=True, exist_ok=True)

    ui_files = sorted(src.glob("*.ui"))
    if not ui_files:
        print(f"[gen-ui] No .ui files found in {src.resolve()}")
        return 0

    # Generate ui_*.py files
    for ui in ui_files:
        py_name = f"ui_{ui.stem}.py"
        dst = out / py_name
        cmd = ["pyuic6", "-o", str(dst), str(ui)]
        print("[gen-ui] $", " ".join(cmd))
        subprocess.check_call(cmd)  # raises on failure

    # Generate ui_*.pyi files
    for ui in ui_files:
        py_name = f"ui_{ui.stem}.pyi"
        dst = out / py_name
        cmd = ["python3", str(here / "ui2stub.py"), "-o", str(dst), str(ui)]
        print("[ui2stub.py] $", " ".join(cmd))
        subprocess.check_call(cmd)  # raises on failure

    print(f"[gen-ui] Generated {2 * len(ui_files)} file(s) in {out.resolve()}")
    return 0



def main() -> None:
    # Optional: take directories from argv: uv run gen-ui ui generated
    ui_dir = sys.argv[1] if len(sys.argv) > 1 else "src/magboltz_gui/ui"
    out_dir = sys.argv[2] if len(sys.argv) > 2 else "src/magboltz_gui/generated"
    raise SystemExit(compile_ui(ui_dir, out_dir))

main()


def linspace_0_1(n: int) -> list[float]:
    "Generate n evenly spaced values between 0 and 1 inclusive."""
    if n <= 1:
        return [0.0]
    step = 1.0 / (n - 1)
    return [i * step for i in range(n)]


from pathlib import Path

from magboltz_gui.data.input_cards import InputCards, InputGas


def load(filename: Path) -> InputCards:
    with filename.open("r") as f:
        # Card 1
        line = f.readline()
        parts = line.strip().split()
        num_gases = int(parts[0])
        number_of_real_collisions = int(parts[1])
        enable_penning = bool(parts[2] == "1")
        enable_thermal = bool(parts[3] == "1")
        final_energy = float(parts[4])

        # Card 2
        line = f.readline()
        gas_ids = [int(x) for x in line.strip().split()[:num_gases]]

        # Card 3
        line = f.readline()
        parts = line.strip().split()
        gas_fracs = [float(x) for x in parts[:num_gases]]
        gas_temperature = float(parts[6])
        gas_pressure = float(parts[7])

        # Card 4
        line = f.readline()
        parts = line.strip().split()
        electric_field = float(parts[0])
        magnetic_field = float(parts[1])
        angle = float(parts[2])

    # Rebuild gases list
    gases = [InputGas(gas_id=gas_ids[i], gas_frac=gas_fracs[i]) for i in range(num_gases)]

    return InputCards(
        gases=gases,
        number_of_real_collisions=number_of_real_collisions,
        enable_penning=enable_penning,
        enable_thermal=enable_thermal,
        final_energy=final_energy,
        gas_temperature=gas_temperature,
        gas_pressure=gas_pressure,
        electric_field=electric_field,
        magnetic_field=magnetic_field,
        angle=angle,
    )


def save(input_cards: InputCards, filename: Path) -> None:

    gas_ids = [gas.gas_id for gas in input_cards.gases] + [80] * (6 - len(input_cards.gases))
    gas_fracs = [gas.gas_frac for gas in input_cards.gases] + [0.0] * (6 - len(input_cards.gases))

    with filename.open("w") as f:
        # Card 1
        f.write(
            f"{len(input_cards.gases)}\t{input_cards.number_of_real_collisions}\t{'1' if input_cards.enable_penning else '0'}\t{'1' if input_cards.enable_thermal else '0'}\t{input_cards.final_energy}\n"
        )

        # Card 2
        f.write("\t".join(str(v) for v in gas_ids) + "\n")

        # Card 3
        f.write("\t".join(str(v) for v in gas_fracs))
        f.write(f"\t{input_cards.gas_temperature}\t{input_cards.gas_pressure}\n")

        # Card 4
        f.write(f"{input_cards.electric_field}\t{input_cards.magnetic_field}\t{input_cards.angle}\n")

        # Card 4N + 1
        f.write("{} {} {} {} {}\n".format(0, 0, 0, 0, 0.0))



from dataclasses import dataclass, field
from typing import List, Optional


@dataclass
class InputGas:

    gas_id: int
    # NGAS etc.: gas number identifiers (between 1 and 80) see gas list below for identifying numbers.

    gas_frac: float
    # FRAC etc.: percentage fraction of gas1 etc.;


@dataclass
class InputCards:

    @property
    def number_of_gases(self) -> int:
        # NGAS: number of gases in mixture
        return len(self.gases)

    number_of_real_collisions: int = 5
    # NMAX: number of real collisions (multiple of 10**7), use a value between 2 and 5 for inelastic gas to obtain 1 % accuracy, use a value above 10 for better than 0.5 % accuracy and a value of at least 10 for pure elastic gases like Argon

    enable_penning: bool = False
    # IPEN   = 0 Penning effects not included, 1 Penning effects included (see instructions above)

    enable_thermal: bool = True
    # ITHRM  = 0 gas motion assumed to be at 0 Kelvin (static gas)
    # ITHRM  = 1 gas motion taken to be at input temperature

    final_energy: float = 0.0
    # EFINAL: upper limit of the electron energy in electron Volts, if EFINAL = 0.0, program automatically calculates upper integration energy limit

    gas_temperature: float = 20.0
    # TEMP: temperature of gas in centigrade

    gas_pressure: float = 760.0
    # TORR: pressure of gas in Torr

    electric_field: float = 1000.0
    # EFIELD: electric field in Volt/cm

    magnetic_field: float = 0.0
    # BMAG: magnitude of the magnetic field in kilogauss

    angle: float = 0.0
    # BTHETA: angle between the electric and magnetic fields in degrees

    gases: List[InputGas] = field(default_factory=list)



# gas_pie_widget.py
from __future__ import annotations
from typing import Dict
from PyQt6 import QtWidgets
from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt import NavigationToolbar2QT

from matplotlib.figure import Figure


class GasPieWidget(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._figure = Figure(figsize=(4, 4))
        self._canvas = FigureCanvas(self._figure)  # type:ignore
        self._ax = self._figure.add_subplot(111)
        self._ax.set_aspect("equal")  # keep it round

        toolbar = NavigationToolbar2QT(self._canvas, self)  # type:ignore

        layout = QtWidgets.QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(toolbar)
        layout.addWidget(self._canvas)

    def set_composition(
        self, composition: Dict[str, float], title: str = "Gas composition", normalize: bool = False
    ) -> None:
        if not composition:
            raise ValueError("composition dict is empty")
        labels = list(composition.keys())
        values = [float(v) for v in composition.values()]
        if any(v < 0 for v in values):
            raise ValueError("values must be non-negative")
        total = sum(values)
        if total <= 0:
            raise ValueError("sum must be > 0")
        if normalize:
            values = [v / total for v in values]

        self._ax.clear()
        self._ax.set_title(title)
        # autopct shows percentages; pctdistance/labeldistance tweak spacing for tiny slices
        self._ax.pie(
            values, labels=labels, autopct=lambda p: f"{p:.1f}%", startangle=90, pctdistance=0.75, labeldistance=1.05
        )
        self._ax.set_aspect("equal")
        self._canvas.draw_idle()  # type:ignore



from typing import Any
from PyQt6 import QtCore, QtGui, QtWidgets

class Ui_Form:
    Form: QtWidgets.QWidget
    buttonBox: QtWidgets.QDialogButtonBox
    gridLayout: QtWidgets.QGridLayout
    groupBox: QtWidgets.QGroupBox
    label: QtWidgets.QLabel
    lineEdit: QtWidgets.QLineEdit
    toolButton: QtWidgets.QToolButton
    verticalLayout: QtWidgets.QVBoxLayout
    def setupUi(self, widget: QtWidgets.QWidget) -> None: ...
    def retranslateUi(self, widget: QtWidgets.QWidget) -> None: ...


from csv import DictReader
from dataclasses import dataclass, field
from pathlib import Path
from typing import List, Optional


@dataclass
class GasItem:

    id: int
    name: str
    formula: Optional[str]
    year: Optional[int]
    note: Optional[str]
    rating: Optional[int]

    @property
    def as_name(self) -> str:
        return self.name

    @property
    def as_name_formula(self) -> str:
        return f"{self.name} ({self.formula})" if self.formula else self.name

    @property
    def as_formula(self) -> str:
        return self.formula if self.formula else self.name


@dataclass
class GasDatabase:
    content: List[GasItem] = field(default_factory=list)

    def load(self, filepath: Path) -> None:
        with open(filepath, newline="", encoding="utf-8") as csvfile:
            reader = DictReader(csvfile)
            for row in reader:
                gas = GasItem(
                    id=int(row["id"]),
                    name=row["name"].strip(),
                    formula=row["formula"].strip() if row["formula"] else None,
                    year=int(row["year"]) if row["year"] else None,
                    note=row["note"].strip() if row["note"] else None,
                    rating=int(row.get("rating", "")) if row["rating"] else None,
                )
                self.content.append(gas)

    def get(self, id: int) -> GasItem:

        for item in self.content:
            if item.id == id:
                return item

        raise KeyError(f"Gas with id {id} not found")



from __future__ import annotations
import platform
from importlib.resources import files
from pathlib import Path
from typing import Optional, List, Tuple, Callable

from PyQt6.QtCore import Qt, QPoint
from PyQt6.QtGui import QIcon, QAction
from PyQt6.QtWidgets import (
    QMainWindow,
    QFileDialog,
    QMessageBox,
    QTableWidgetItem,
    QHeaderView,
    QTableWidget,
    QApplication,
    QVBoxLayout,
)
from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg
from matplotlib.colors import Colormap, ListedColormap
from matplotlib.figure import Figure
from matplotlib.pyplot import colormaps, get_cmap
from matplotlib.rcsetup import cycler

from magboltz_gui.data.database import GasDatabase
from magboltz_gui.data.input_cards import InputCards, InputGas
from magboltz_gui.generated.ui_main import Ui_MainWindow
from magboltz_gui.util import parser
from magboltz_gui.util.other import linspace_0_1
from magboltz_gui.window.delegates import PercentDelegate, GasNameDelegate
from magboltz_gui.util.process import ProcessManager


class MagboltzGUI(QMainWindow, Ui_MainWindow):

    def __init__(self) -> None:
        super().__init__()
        self.setupUi(self)
        # uic.loadUi(files("magboltz_gui.ui").joinpath("main.ui"), self) <- Not needed anymore because we use pyuic6

        system = platform.system()
        if system == "Darwin":
            self.initDarwinActionIcons()

        self.centralWidget().setVisible(False)

        self._currentInputFile: Optional[Path] = None
        self._currentResultFile: Optional[Path] = None

        self._currentCards: InputCards = InputCards()
        self._currentModified: bool = False


        # Associating button to actions
        self.btnGasAdd.setDefaultAction(self.actionGasAdd)
        self.btnGasRemove.setDefaultAction(self.actionGasRemove)
        self.btnGasNormalize.setDefaultAction(self.actionGasNormalize)
        self.btnCmdCopyToClipbord.setDefaultAction(self.actionCmdCopyToClipboard)
        self.btnResultSave.setDefaultAction(self.actionResultSave)
        self.btnResultExport.setDefaultAction(self.actionResultExport)
        self.btnGraphSave.setDefaultAction(self.actionGraphSave)
        self.btnGasUp.setDefaultAction(self.actionGasMoveUp)
        self.btnGasDown.setDefaultAction(self.actionGasMoveDown)

        # Associating actions to methods
        self.actionNew.triggered.connect(self.fileNew)
        self.actionOpen.triggered.connect(self.fileOpen)
        self.actionSave.triggered.connect(self.fileSave)
        self.actionSaveAs.triggered.connect(self.fileSaveAs)
        self.actionClose.triggered.connect(self.fileClose)
        self.actionRun.triggered.connect(self.run)
        self.actionGasAdd.triggered.connect(self.gasAdd)
        self.actionGasRemove.triggered.connect(self.gasRemove)
        self.actionGasNormalize.triggered.connect(self.gasNormalize)
        self.actionCmdCopyToClipboard.triggered.connect(self.cmdCopyToClipboard)
        self.actionResultSave.triggered.connect(self.saveResult)
        self.actionResultExport.triggered.connect(self.openExportWindow)
        self.actionGasMoveUp.triggered.connect(self.gasMoveUp)
        self.actionGasMoveDown.triggered.connect(self.gasMoveDown)
        self.actionGraphSave.triggered.connect(self.graphSave)

        self.mainTab.setCurrentWidget(self.tabConfiguration)

        self.fillColorMap()

        self.createPieChart()

        self.cmbLabelFormat.currentIndexChanged.connect(self.refresh_pie)
        self.cmbColorMap.currentIndexChanged.connect(self.refresh_pie)

        # Make "Gas name" stretch to fill available space
        header = self.gasListTable.horizontalHeader()
        header.setSectionResizeMode(0, QHeaderView.ResizeMode.ResizeToContents)  # Gas ID shrinks to content
        header.setSectionResizeMode(1, QHeaderView.ResizeMode.Stretch)  # Gas name fills extra space
        header.setSectionResizeMode(2, QHeaderView.ResizeMode.ResizeToContents)  # Gas fraction shrinks to content

        header.setMinimumSectionSize(50)

        self.database = GasDatabase()
        self.database.load(Path(str(files("magboltz_gui.database").joinpath("database.csv"))))

        self.magboltzPath: Optional[Path] = None
        self.processes: List[ProcessManager] = []
        self.actionNew.trigger()

    def createPieChart(self) -> None:
        fig = Figure(figsize=(3, 3))
        canvas = FigureCanvasQTAgg(fig)  # type:ignore

        # 2. Create an Axes and draw something (pie chart)
        ax = fig.add_subplot(111)
        ax.pie([], labels=[], autopct="%1.1f%%")

        # 3. Add the canvas to the QWidget container
        layout = QVBoxLayout(self.pieContainer)
        layout.setContentsMargins(0, 0, 0, 0)  # remove spacing
        layout.addWidget(canvas)

        self._gas_fig = fig
        self._gas_ax = ax
        self._gas_canvas = canvas

        self.splitterGases.setSizes([600, 200])

    def initDarwinActionIcons(self) -> None:
        # This method set the icons for macOS

        actions: List[QAction] = [
            self.actionNew,
            self.actionOpen,
            self.actionSave,
            self.actionSaveAs,
            self.actionRevert,
            self.actionClose,
            self.actionQuit,
            self.actionRun,
            self.actionGasAdd,
            self.actionGasRemove,
            self.actionGasNormalize,
            self.actionCmdCopyToClipboard,
            self.actionResultExport,
        ]

        icns_map = {
            self.actionNew: "src/magboltz_gui/icons/macOS_icons/new_file.png",
            self.actionOpen: "src/magboltz_gui/icons/macOS_icons/open.png",
            self.actionSave: "src/magboltz_gui/icons/macOS_icons/save.png",
            self.actionSaveAs: "src/magboltz_gui/icons/macOS_icons/save_as.png",
            self.actionRevert: "src/magboltz_gui/icons/macOS_icons/revert.png",
            self.actionClose: "src/magboltz_gui/icons/macOS_icons/TrashIcon.icns",
            self.actionQuit: "src/magboltz_gui/icons/macOS_icons/AlertStopIcon.icns",
            self.actionRun: "src/magboltz_gui/icons/macOS_icons/execute.png",
            self.actionGasAdd: "src/magboltz_gui/icons/macOS_icons/add.png",
            self.actionGasRemove: "src/magboltz_gui/icons/macOS_icons/remove.png",
            self.actionGasNormalize: "src/magboltz_gui/icons/macOS_icons/CalculatorIcon.icns",
            self.actionCmdCopyToClipboard: "src/magboltz_gui/icons/macOS_icons/copy.png", 
            self.actionResultExport: "src/magboltz_gui/icons/macOS_icons/export.png", 
        }

        for action in actions:
            icon = icns_map.get(action, None)
            if icon is not None:
                if Path(icon).is_file():
                    action.setIcon(QIcon(icon))

    def fileNew(self) -> None:
        self._currentCards = InputCards()
        self._currentCards = InputCards()
        self._currentInputFile = None
        self._currentModified = False
        self._currentResultFile = None
        self.connect()

    def fileOpen(self) -> None:

        file_name, _ = QFileDialog.getOpenFileName(self, "Open File", "", "All Files (*);;Text Files (*.txt)")

        if file_name:  # If user picked a file (not Cancel)
            self._currentCards = parser.load(Path(file_name))
            self._currentInputFile = Path(file_name)
            self._currentModified = False
            self.updateCmdLine()
            self.connect()

    def fileSaveAs(self) -> None:

        if not self.checkInputOpen():
            return

        file_name, _ = QFileDialog.getSaveFileName(
            self,
            "Save File",
            str(self._currentInputFile) if self._currentInputFile else "input.txt",
            "All Files (*);;Text Files (*.txt)",
        )

        if file_name:
            parser.save(self._currentCards, Path(file_name))
            self._currentInputFile = Path(file_name)
            self._currentModified = False
            self.updateCmdLine()

    def fileSave(self) -> None:
        if not self.checkInputOpen():
            return

        if self._currentInputFile is None:
            self.fileSaveAs()
        else:
            parser.save(self._currentCards, self._currentInputFile)
            self._currentModified = False
            self.updateCmdLine()

    def fileClose(self) -> None:
        if self._currentCards is not None:

            response = self.show_save_question()

            if response is None:
                return

            if response == True:
                self.fileSave()

            self._currentInputFile = None
            self._currentResultFile = None

            self._currentCards = InputCards()
            self._currentModified = False

            self.updateCmdLine()
            self.disconnect()

    def cmdCopyToClipboard(self) -> None:
        QApplication.clipboard().setText(self.commandLine.text())

    def checkInputOpen(self) -> bool:
        if self._currentCards is None:
            self.show_error("Error", "File was not open")
            return False
        return True

    def saveResult(self) -> None:

        if not self.checkInputOpen():
            return

        if not self.consoleOutput.toPlainText().strip():
            self.show_info("File Saved", f"First run Magboltz to save the results")

            return

        currentResultFileName, _ = QFileDialog.getSaveFileName(
            self,
            "Save Result",
            str(self._currentResultFile) if self._currentResultFile else "output.txt",
            "All Files (*);;Text Files (*.txt)",
        )

        if currentResultFileName is not None:

            currentResultFile = Path(currentResultFileName)

            text = self.consoleOutput.toPlainText()

            try:
                with open(currentResultFile, "w", encoding="utf-8") as file:
                    file.write(text)
            except Exception as e:
                self.show_error("Error", f"Could not save result: {e}")

            else:
                self.show_info("Success", f"File saved successfully:\n{currentResultFile}")
                self._currentResultFile = currentResultFile

    def openExportWindow(self) -> None:
        print("Test Test")

    def run(self) -> None:

        if self._currentCards is None:
            self.show_error("To run the magboltz process, you must to open the input file")
            return

        if self._currentInputFile is None:
            self.show_error("Error", "To run the magboltz process, you must to save the file")
            return

        if self._currentModified is False:
            self.fileSave()

        self.mainTab.setCurrentWidget(self.tabExecution)

        process = ProcessManager(self)
        self.processes.append(process)
        process.run()

    def updateCmdLine(self) -> None:

        if self._currentInputFile is not None:
            self.commandLine.setText(f"{self.magboltzPath or 'magboltz'} < {self._currentInputFile}")
        else:
            self.commandLine.setText("")

    def onFinalEnergyAutoChanged(self, state: int) -> None:
        if Qt.CheckState(state) == Qt.CheckState.Checked:
            self.spinFinalEnergy.setValue(0.0)
        else:
            self.spinFinalEnergy.setValue(50.0)

    def show_error(
        self,
        title: str,
        message: str,
        buttons: QMessageBox.StandardButton = QMessageBox.StandardButton.Ok,
        default: QMessageBox.StandardButton | None = None,
    ) -> None:
        self.show_message(QMessageBox.Icon.Critical, title, message, buttons, default)

    def show_info(
        self,
        title: str,
        message: str,
        buttons: QMessageBox.StandardButton = QMessageBox.StandardButton.Ok,
        default: QMessageBox.StandardButton | None = None,
    ) -> None:
        self.show_message(QMessageBox.Icon.Information, title, message, buttons, default)

    def show_message(
        self,
        icon: QMessageBox.Icon,
        title: str,
        message: str,
        buttons: QMessageBox.StandardButton = QMessageBox.StandardButton.Ok,
        default: QMessageBox.StandardButton | None = None,
    ) -> None:

        msg = QMessageBox(self)
        msg.setIcon(icon)
        msg.setWindowTitle(title)
        msg.setText(message)

        # Buttons
        msg.setStandardButtons(buttons)
        if default is not None:
            msg.setDefaultButton(default)

        # Resize automatically based on content
        msg.setTextInteractionFlags(Qt.TextInteractionFlag.TextSelectableByMouse)
        msg.adjustSize()

        # Center relative to parent window
        parent_geom = self.geometry()
        msg_geom = msg.frameGeometry()

        x = parent_geom.center().x() - msg_geom.width() // 2
        y = parent_geom.center().y() - msg_geom.height() // 2
        msg.move(x, y)

        msg.exec()

    def show_save_question(self) -> Optional[bool]:
        reply = QMessageBox.question(
            self,
            "Save Changes?",
            "Do you want to save changes before closing?",
            QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No | QMessageBox.StandardButton.Cancel,
            QMessageBox.StandardButton.Yes,
        )

        if reply == QMessageBox.StandardButton.Yes:
            return True
        elif reply == QMessageBox.StandardButton.No:
            return False
        else:
            return None

    def connect(self) -> None:

        self.centralWidget().setVisible(True)

        self.spinRealInteractions.setValue(self._currentCards.number_of_real_collisions)
        self.checkPenning.setChecked(self._currentCards.enable_penning)
        self.checkThermal.setChecked(self._currentCards.enable_thermal)
        self.spinFinalEnergy.setValue(self._currentCards.final_energy)
        self.checkFinalEnergyAuto.setChecked(self._currentCards.final_energy == 0.0)
        self.spinGasTemperature.setValue(self._currentCards.gas_temperature)
        self.spinGasPressure.setValue(self._currentCards.gas_pressure)
        self.spinElectricField.setValue(self._currentCards.electric_field)
        self.spinMagneticField.setValue(self._currentCards.magnetic_field)
        self.spinAngle.setValue(self._currentCards.angle)

        self.spinRealInteractions.valueChanged.connect(self.onRealInteractionsChanged)
        self.checkPenning.stateChanged.connect(self.onPenningChanged)
        self.checkThermal.stateChanged.connect(self.onThermalChanged)
        self.checkFinalEnergyAuto.stateChanged.connect(self.onFinalEnergyAutoChanged)
        self.spinFinalEnergy.valueChanged.connect(self.onFinalEnergyChanged)
        self.spinGasTemperature.valueChanged.connect(self.onGasTemperatureChanged)
        self.spinGasPressure.valueChanged.connect(self.onGasPressureChanged)
        self.spinElectricField.valueChanged.connect(self.onElectricFieldChanged)
        self.spinMagneticField.valueChanged.connect(self.onMagneticFieldChanged)
        self.spinAngle.valueChanged.connect(self.onAngleChanged)

        self.gasListTable.cellChanged.connect(self.refresh_pie)

        gas_name_delegate = GasNameDelegate(self, self.gasListTable)
        self.gasListTable.setItemDelegateForColumn(1, gas_name_delegate)

        delegate = PercentDelegate(self, self.gasListTable)
        self.gasListTable.setItemDelegateForColumn(2, delegate)

        self.refresh()

    def get_colors(self, cmap_name: str, n: int) -> List[Tuple[float, float, float, float]]:
        cmap = get_cmap(cmap_name)

        # print("===============================")
        # print("Name:", cmap.name)
        # print("Type:", type(cmap))
        # print("N:", cmap.N)
        # print("Has colors:", hasattr(cmap, "colors"))
        # print("First color:", cmap.colors[0])
        # print("Bad color:", cmap._rgba_bad)

        if isinstance(cmap, ListedColormap):
            if cmap.N < 256:
                return list(cmap.colors) #type: ignore
            else:
                return list(cmap.resampled(n).colors) #type: ignore
        else:
            return [cmap(i / (n - 1 if n > 1 else 1)) for i in range(n)]

    def refresh_pie(self) -> None:

        self._gas_ax.clear()

        gas_fracs = []
        gas_labels = []

        for gas in self._currentCards.gases:


            try:
                db_item = self.database.get(gas.gas_id)

                match self.cmbLabelFormat.currentIndex():
                    case 0:
                        gas_name = db_item.as_formula
                    case 1:
                        gas_name = db_item.as_name
                    case 2:
                        gas_name = db_item.as_name_formula
                    case 3:
                        gas_name = ''
                    case _:
                        gas_name = db_item.as_formula
            except KeyError:
                gas_name = "?"

            gas_fracs.append(gas.gas_frac)
            gas_labels.append(gas_name)


        if len(gas_fracs) > 0:
            colors = self.get_colors(self.cmbColorMap.currentText(), len(gas_fracs))
            self._gas_ax.set_prop_cycle(cycler(color=colors))

        self._gas_ax.pie(
            gas_fracs if sum(gas_fracs) > 0 else [],
            labels=gas_labels if sum(gas_fracs) > 0 else [],
            autopct="%1.1f%%",
            wedgeprops={"edgecolor": "black", "linewidth": 1},

        )

        self._gas_ax.set_aspect("equal")
        self._gas_canvas.draw()  # type:ignore

    def refresh(self) -> None:

        self.gasListTable.clear()
        self.gasListTable.setRowCount(0)

        self.gasListTable.setHorizontalHeaderLabels(["Gas ID", "Gas name", "Gas fraction"])

        self.gasListTable.verticalHeader().setVisible(False)
        self.gasListTable.setSelectionBehavior(QTableWidget.SelectionBehavior.SelectRows)
        self.gasListTable.setSelectionMode(QTableWidget.SelectionMode.SingleSelection)
        self.gasListTable.setShowGrid(False)

        i = 0
        for gas in self._currentCards.gases:
            gas_id_widget = QTableWidgetItem(str(gas.gas_id))
            gas_id_widget.setTextAlignment(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)
            gas_id_widget.setFlags(gas_id_widget.flags() & ~Qt.ItemFlag.ItemIsEditable)

            try:
                gas_name = self.database.get(gas.gas_id).as_name_formula
            except KeyError:
                gas_name = "(select gas)"

            gas_name_widget = QTableWidgetItem(gas_name)

            gas_frac_widget = QTableWidgetItem(f"{gas.gas_frac} %")
            gas_frac_widget.setTextAlignment(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)

            self.gasListTable.insertRow(i)
            self.gasListTable.setItem(i, 0, gas_id_widget)
            self.gasListTable.setItem(i, 1, gas_name_widget)
            self.gasListTable.setItem(i, 2, gas_frac_widget)
            i += 1

        self.refresh_pie()
        self.refreshNeedNormalize()

    def refreshNeedNormalize(self) -> None:
        sum = 0.
        for gas in self._currentCards.gases:
            sum += gas.gas_frac

        self.lblNeedNormalize.setVisible(sum < 99.9 or sum >= 100.1)

    def gasAdd(self) -> None:

        row_index = self.gasListTable.rowCount()
        self.gasListTable.insertRow(row_index)
        self._currentCards.gases.append(InputGas(80, 0.0))
        self.refresh()
        self.gasListTable.setCurrentCell(row_index, 0)

    def gasRemove(self) -> None:
        row = self.gasListTable.currentRow()

        if row >= 0:
            self._currentCards.gases.pop(row)
            self.refresh()
        else:
            self.show_error("Error", "Select a row to remove")

    def gasNormalize(self) -> None:
        fraction_sum = 0.0
        for gas in self._currentCards.gases:
            fraction_sum += gas.gas_frac

        if fraction_sum > 0.0:

            for gas in self._currentCards.gases:
                gas.gas_frac = round(gas.gas_frac * 100. / fraction_sum, 2)
        else:
            # Split evenly
            for gas in self._currentCards.gases:
                gas.gas_frac = round(100. / len(self._currentCards.gases), 2)

        self.refresh()

    def gasMoveUp(self) -> None:

        row = self.gasListTable.currentRow()

        if row >= 0:
            if row > 0:
                self._currentCards.gases[row], self._currentCards.gases[row - 1] = self._currentCards.gases[row - 1], self._currentCards.gases[row]
                self.refresh()
                self.gasListTable.setCurrentCell(row - 1, 0)
        else:
            self.show_error("Error", "Select a row to move")

    def gasMoveDown(self) -> None:

        row = self.gasListTable.currentRow()

        if row >= 0:
            if row < len(self._currentCards.gases) - 1:
                self._currentCards.gases[row], self._currentCards.gases[row + 1] = self._currentCards.gases[row + 1], self._currentCards.gases[row]
                self.refresh()
                self.gasListTable.setCurrentCell(row + 1, 0)
        else:
            self.show_error("Error", "Select a row to move")

    def graphSave(self) -> None:
        canvas = self._gas_fig.canvas
        types = canvas.get_supported_filetypes()
        print(types)

        items: List[Tuple[str, str]] = []
        for ext, desc in sorted(types.items()):
            items.append((f"{desc} (*.{ext})", ext))


        dict_items = { g : [k[1] for k in items if k[0].startswith(g)] for g in set(' '.join(v[0].split()[:-1]) for v in items)}
        dict_items["All files"] = []

        fn_format_ext : Callable[[str], str] = lambda x: f"*.{x}" if x else "*"

        group_items = [ (format_name + " (" + ' '.join(fn_format_ext(x) for x in format_exts) + ")", ' '.join(format_exts)) for format_name, format_exts in dict_items.items() ]

        filter_str = ";;".join(name for name, _ in group_items)

        path_str, selected = QFileDialog.getSaveFileName(
            self,
            "Save chart",
            "",
            filter_str
        )

        if not path_str:
            return

        path = Path(path_str)

        # Decide the format/extension
        ext = path.suffix.lower().lstrip(".")
        if not ext:
            # No extension typed → infer from selected filter (take its mapped ext)
            for name, mapped_ext in group_items:
                if name == selected:
                    ext = mapped_ext
                    break
            # If still unknown, default to PNG
            if not ext:
                ext = "png"
            path = path.with_suffix(f".{ext}")

        # Save with tight bbox, decent DPI
        self._gas_fig.savefig(
            path,
            format=ext,
            dpi=300,
            bbox_inches="tight",
            facecolor=self._gas_fig.get_facecolor()  # keep current background
        )

    def onRealInteractionsChanged(self, value: int) -> None:

        self._currentCards.number_of_real_collisions = value

    def onPenningChanged(self, value: bool) -> None:

        self._currentCards.enable_penning = value

    def onThermalChanged(self, value: bool) -> None:

        self._currentCards.enable_thermal = value

    def onFinalEnergyChanged(self, value: float) -> None:

        self.checkFinalEnergyAuto.setChecked(value == 0.0)
        self._currentCards.final_energy = value

    def onGasTemperatureChanged(self, value: float) -> None:

        self._currentCards.gas_temperature = value

    def onGasPressureChanged(self, value: float) -> None:

        self._currentCards.gas_pressure = value

    def onElectricFieldChanged(self, value: float) -> None:

        self._currentCards.electric_field = value

    def onMagneticFieldChanged(self, value: float) -> None:

        self._currentCards.magnetic_field = value

    def onAngleChanged(self, value: float) -> None:

        self._currentCards.angle = value

    def disconnect(self) -> None:

        self.centralWidget().setVisible(False)

        self.spinRealInteractions.valueChanged.disconnect(self.onRealInteractionsChanged)
        self.checkPenning.stateChanged.disconnect(self.onPenningChanged)
        self.checkThermal.stateChanged.disconnect(self.onThermalChanged)
        self.checkFinalEnergyAuto.stateChanged.disconnect(self.onFinalEnergyAutoChanged)
        self.spinFinalEnergy.valueChanged.disconnect(self.onFinalEnergyChanged)
        self.spinGasTemperature.valueChanged.disconnect(self.onGasTemperatureChanged)
        self.spinGasPressure.valueChanged.disconnect(self.onGasPressureChanged)
        self.spinElectricField.valueChanged.disconnect(self.onElectricFieldChanged)
        self.spinMagneticField.valueChanged.disconnect(self.onMagneticFieldChanged)
        self.spinAngle.valueChanged.disconnect(self.onAngleChanged)

        # self.btnGasAdd.triggered.disconnect(self.onBtnGasAdd)
        # self.btnGasRemove.triggered.disconnect(self.onBtnGasRemove)
        # self.btnGasNormalize.triggered.disconnect(self.onBtnGasNormalize)
        # self.btnExport.triggered.disconnect(self.onBtnExport)

        self.gasListTable.clear()

    def fillColorMap(self) -> None:
        self.cmbColorMap.clear()

        names = [name for name in colormaps() if hasattr(get_cmap(name), "colors")]

        names.sort()

        #self.cmbColorMap.setIconSize(self.cmbColorMap.iconSize())  # keep default or adjust via setIconSize()
        for name in names:
            self.cmbColorMap.addItem(name, userData=name)



from __future__ import annotations
from typing import Optional, TYPE_CHECKING
from typing_extensions import override


from PyQt6.QtCore import Qt, QModelIndex, QAbstractItemModel, QEvent
from PyQt6.QtWidgets import QDoubleSpinBox, QStyledItemDelegate, QWidget, QStyleOptionViewItem, QTableWidget

from magboltz_gui.window.search_window import SearchWindow

if TYPE_CHECKING:
    from magboltz_gui.window.main_window import MagboltzGUI


class PercentSpinBox(QDoubleSpinBox):
    def __init__(self, parent: Optional[QWidget] = None) -> None:
        super().__init__(parent)
        self.setSuffix(" %")
        self.setDecimals(1)
        self.setRange(0, 100)
        self.setSingleStep(0.5)
        self.setAlignment(Qt.AlignmentFlag.AlignRight)


class PercentDelegate(QStyledItemDelegate):

    def __init__(self, main_window: MagboltzGUI, parent: Optional[QWidget] = None) -> None:
        super().__init__(parent)
        self._main_window = main_window

    def createEditor(self, parent: QWidget, option: QStyleOptionViewItem, index: QModelIndex) -> QWidget:
        editor = QDoubleSpinBox(parent)
        editor.setSuffix(" %")
        editor.setDecimals(1)
        editor.setRange(0, 100)
        editor.setSingleStep(0.5)
        editor.setAlignment(Qt.AlignmentFlag.AlignRight)
        return editor

    @override
    def setEditorData(self, editor: QWidget, index: QModelIndex) -> None:
        assert isinstance(editor, QDoubleSpinBox)
        text = index.model().data(index, Qt.ItemDataRole.EditRole)
        value = float(str(text).replace(" %", "").strip())
        editor.setValue(value)

    @override
    def setModelData(self, editor: QWidget, model: QAbstractItemModel, index: QModelIndex) -> None:
        assert isinstance(editor, QDoubleSpinBox)
        editor.interpretText()
        value = editor.value()
        model.setData(index, f"{value:.1f} %", Qt.ItemDataRole.EditRole)

        self._main_window._currentCards.gases[index.row()].gas_frac = value
        self._main_window.refresh()


class GasNameDelegate(QStyledItemDelegate):

    def __init__(self, main_window: MagboltzGUI, parent: Optional[QWidget] = None):
        super().__init__(parent)
        self._main_window = main_window

    @override
    def createEditor(
        self,
        parent: QWidget,
        option: QStyleOptionViewItem,
        index: QModelIndex,
    ) -> QWidget:
        # DO NOT return an editor. We’ll use a popup instead.
        return QWidget(parent)

    @override
    def setEditorData(self, editor: QWidget, index: QModelIndex) -> None:
        pass  # Not needed

    @override
    def setModelData(
        self,
        editor: QWidget,
        model: QAbstractItemModel,
        index: QModelIndex,
    ) -> None:
        pass  # Not needed

    @override
    def editorEvent(
        self,
        event: QEvent,
        model: QAbstractItemModel,
        option: QStyleOptionViewItem,
        index: QModelIndex,
    ) -> bool:
        if event.type() == event.Type.MouseButtonDblClick:
            # Create and show the SearchWidget as a modal dialog
            search_dialog = SearchWindow(self._main_window.database)
            if search_dialog.exec():  # exec() returns QDialog.Accepted if OK pressed

                selected_gas_id = search_dialog.getSelectedGasId()
                # Update Gas ID column (assume column 0 for Gas ID)
                gas_id_index = index.siblingAtColumn(0)
                model.setData(gas_id_index, selected_gas_id, Qt.ItemDataRole.EditRole)

                self._main_window._currentCards.gases[index.row()].gas_id = selected_gas_id
                self._main_window.refresh()
                self._main_window.gasListTable.setCurrentCell(index.row(), 0)
            return True
        return False



from typing import Any
from PyQt6 import QtCore, QtGui, QtWidgets

class Ui_Dialog:
    Dialog: QtWidgets.QDialog
    buttonBox: QtWidgets.QDialogButtonBox
    groupBox: QtWidgets.QGroupBox
    label: QtWidgets.QLabel
    line: QtWidgets.QFrame
    line_2: QtWidgets.QFrame
    radioButton: QtWidgets.QRadioButton
    radioButton_2: QtWidgets.QRadioButton
    verticalLayout: QtWidgets.QVBoxLayout
    verticalLayout_2: QtWidgets.QVBoxLayout
    verticalLayout_3: QtWidgets.QVBoxLayout
    verticalSpacer: QtWidgets.QSpacerItem
    def setupUi(self, widget: QtWidgets.QDialog) -> None: ...
    def retranslateUi(self, widget: QtWidgets.QDialog) -> None: ...


# Form implementation generated from reading ui file 'src/magboltz_gui/ui/export.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(1034, 717)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(parent=Dialog)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.radioButton = QtWidgets.QRadioButton(parent=self.groupBox)
        self.radioButton.setObjectName("radioButton")
        self.verticalLayout_3.addWidget(self.radioButton)
        self.line = QtWidgets.QFrame(parent=self.groupBox)
        self.line.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_3.addWidget(self.line)
        self.label = QtWidgets.QLabel(parent=self.groupBox)
        self.label.setEnabled(True)
        self.label.setObjectName("label")
        self.verticalLayout_3.addWidget(self.label)
        self.line_2 = QtWidgets.QFrame(parent=self.groupBox)
        self.line_2.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_3.addWidget(self.line_2)
        self.radioButton_2 = QtWidgets.QRadioButton(parent=self.groupBox)
        self.radioButton_2.setObjectName("radioButton_2")
        self.verticalLayout_3.addWidget(self.radioButton_2)
        self.verticalLayout.addWidget(self.groupBox)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept) # type: ignore
        self.buttonBox.rejected.connect(Dialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.groupBox.setTitle(_translate("Dialog", "Export results in CSV format"))
        self.radioButton.setText(_translate("Dialog", "Summary"))
        self.label.setText(_translate("Dialog", "Detailed collision frequencies:"))
        self.radioButton_2.setText(_translate("Dialog", "Normalised energy distribution"))



# --- Put this at the very top of main.py (before creating QApplication) ---

import os, sys, platform, re
from pathlib import Path
from typing import Tuple, Optional


def _detect_linux_distro() -> str:
    """
    Returns a short distro key: 'debian', 'ubuntu', 'fedora', 'arch', 'opensuse', or 'linux'.
    """
    try:
        data = Path("/etc/os-release").read_text(errors="ignore")
    except Exception:
        return "linux"

    def get(k: str) -> str:
        m = re.search(rf'^{k}=(?:"([^"]+)"|([^\n]+))', data, flags=re.M)
        return (m.group(1) or m.group(2) or "").lower() if m else ""

    id_ = get("ID")
    like = get("ID_LIKE")
    # normalize some families
    if id_ in {"ubuntu"} or "ubuntu" in like:
        return "ubuntu"
    if id_ in {"debian"} or "debian" in like:
        return "debian"
    if id_ in {"fedora"} or "fedora" in like or "rhel" in like or "centos" in like:
        return "fedora"
    if id_ in {"arch", "manjaro", "endeavouros"} or "arch" in like:
        return "arch"
    if id_ in {"opensuse-leap", "opensuse-tumbleweed", "opensuse"} or "suse" in like:
        return "opensuse"
    return "linux"


def _print_install_help_for_missing_pyqt6() -> None:
    system = platform.system()
    if system == "Linux":
        d = _detect_linux_distro()
        print("\nPyQt6 (bindings) is not available.\n", file=sys.stderr)
        if d in {"debian", "ubuntu"}:
            print("Install system packages:", file=sys.stderr)
            print("  sudo apt install python3-pyqt6", file=sys.stderr)
            print("  ", file=sys.stderr)
            print("and be sure to recreate the .venv directory using:", file=sys.stderr)
            print("  uv venv --system-site-packages", file=sys.stderr)
        elif d == "fedora":
            print("Install system packages:", file=sys.stderr)
            print("  sudo dnf install python3-qt6", file=sys.stderr)
            print("  ", file=sys.stderr)
            print("and be sure to recreate the .venv directory using:", file=sys.stderr)
            print("  uv venv --system-site-packages", file=sys.stderr)
        elif d == "arch":
            print("Install system packages:", file=sys.stderr)
            print("  sudo pacman -S python-pyqt6", file=sys.stderr)
            print("  ", file=sys.stderr)
            print("and be sure to recreate the .venv directory using:", file=sys.stderr)
            print("  uv venv --system-site-packages", file=sys.stderr)
        elif d == "opensuse":
            print("Install system packages (adjust python version if needed):", file=sys.stderr)
            print("  sudo zypper install python311-qt6", file=sys.stderr)
            print("  ", file=sys.stderr)
            print("and be sure to recreate the .venv directory using:", file=sys.stderr)
            print("  uv venv --system-site-packages", file=sys.stderr)
        else:
            print("Install your distro’s PyQt6 package.", file=sys.stderr)
            print("Examples:", file=sys.stderr)
            print("  Debian/Ubuntu: sudo apt install python3-pyqt6", file=sys.stderr)
            print("  Fedora:        sudo dnf install python3-qt6", file=sys.stderr)
            print("  ", file=sys.stderr)
            print("and be sure to recreate the .venv directory using:", file=sys.stderr)
            print("  uv venv --system-site-packages", file=sys.stderr)
    elif system == "Darwin":  # macOS
        print("\nPyQt6 is not available on macOS.", file=sys.stderr)
        print("  python3 -m pip install PyQt6 PyQt6-Qt6 PyQt6-Qt6-Data", file=sys.stderr)
        print("Install using Homebrew Qt6:", file=sys.stderr)
        print("  brew install qt@6", file=sys.stderr)
        print("  ", file=sys.stderr)
        print("and be sure to recreate the .venv directory using:", file=sys.stderr)
        print("  uv venv --system-site-packages", file=sys.stderr)
        print("", file=sys.stderr)

    else:  # Windows / other
        print("\nPyQt6 is not available.\nInstall via PyPI:", file=sys.stderr)
        print("  py -m pip install  a\n", file=sys.stderr)
        print("", file=sys.stderr)
        print("  and be sure to recreate the .venv directory using:", file=sys.stderr)
        print("  uv venv --system-site-packages", file=sys.stderr)


def _platform_plugins_ok() -> Tuple[bool, Optional[str]]:
    """
    On Linux: check that at least one platform plugin we can use is present (wayland/xcb).
    On macOS: cocoa is baked in; if import works, we're fine.
    Returns (ok: bool, hint: str|None)
    """
    try:
        from PyQt6.QtCore import QLibraryInfo as LI

        plug_dir = Path(LI.path(LI.LibraryPath.PluginsPath)) / "platforms"
        # plugin filenames vary by OS
        files = [p.name.lower() for p in plug_dir.glob("*")]
        system = platform.system()
        if system == "Linux":
            has_wayland = any("wayland" in f for f in files)

            has_xcb = any("xcb" in f for f in files)
            if has_wayland or has_xcb:
                return True, None
            return False, f"Qt platform plugins not found in: {plug_dir}"
        elif system == "Darwin":
            # Cocoa comes with the Qt framework; if PyQt6 imported, it's generally fine.
            return True, None
        else:
            return True, None
    except Exception as e:
        return False, f"Failed to inspect Qt plugins: {e}"


def ensure_qt_runtime_or_explain() -> None:
    # Prefer Wayland if present, let Qt fall back automatically.
    # os.environ.setdefault("QT_QPA_PLATFORM", "wayland")

    # 1) PyQt6 available?
    try:
        import PyQt6  # noqa: F401
    except Exception as e:
        _print_install_help_for_missing_pyqt6()
        print(f"Details: {e}\n", file=sys.stderr)
        sys.exit(1)



from typing import Any
from PyQt6 import QtCore, QtGui, QtWidgets

class Ui_MainWindow:
    MainWindow: QtWidgets.QMainWindow
    actionClose: QtGui.QAction
    actionCmdCopyToClipboard: QtGui.QAction
    actionGasAdd: QtGui.QAction
    actionGasMoveDown: QtGui.QAction
    actionGasMoveUp: QtGui.QAction
    actionGasNormalize: QtGui.QAction
    actionGasRemove: QtGui.QAction
    actionGraphSave: QtGui.QAction
    actionNew: QtGui.QAction
    actionOpen: QtGui.QAction
    actionQuit: QtGui.QAction
    actionResultExport: QtGui.QAction
    actionResultSave: QtGui.QAction
    actionRevert: QtGui.QAction
    actionRun: QtGui.QAction
    actionSave: QtGui.QAction
    actionSaveAs: QtGui.QAction
    btnCmdCopyToClipbord: QtWidgets.QToolButton
    btnGasAdd: QtWidgets.QToolButton
    btnGasDown: QtWidgets.QToolButton
    btnGasNormalize: QtWidgets.QToolButton
    btnGasRemove: QtWidgets.QToolButton
    btnGasUp: QtWidgets.QToolButton
    btnGraphSave: QtWidgets.QToolButton
    btnResultExport: QtWidgets.QToolButton
    btnResultSave: QtWidgets.QToolButton
    centralwidget: QtWidgets.QWidget
    checkFinalEnergyAuto: QtWidgets.QCheckBox
    checkPenning: QtWidgets.QCheckBox
    checkThermal: QtWidgets.QCheckBox
    cmbColorMap: QtWidgets.QComboBox
    cmbLabelFormat: QtWidgets.QComboBox
    commandLine: QtWidgets.QLineEdit
    consoleOutput: QtWidgets.QTextEdit
    gasListTable: QtWidgets.QTableWidget
    gridLayout: QtWidgets.QGridLayout
    gridLayout_2: QtWidgets.QGridLayout
    gridLayout_3: QtWidgets.QGridLayout
    groupGasList: QtWidgets.QGroupBox
    groupGases: QtWidgets.QGroupBox
    groupSimParameters: QtWidgets.QGroupBox
    horizontalLayout: QtWidgets.QHBoxLayout
    horizontalLayout_2: QtWidgets.QHBoxLayout
    horizontalLayout_3: QtWidgets.QHBoxLayout
    horizontalLayout_4: QtWidgets.QHBoxLayout
    horizontalSpacer: QtWidgets.QSpacerItem
    horizontalSpacer_2: QtWidgets.QSpacerItem
    horizontalSpacer_3: QtWidgets.QSpacerItem
    horizontalSpacer_6: QtWidgets.QSpacerItem
    label: QtWidgets.QLabel
    label_10: QtWidgets.QLabel
    label_11: QtWidgets.QLabel
    label_12: QtWidgets.QLabel
    label_13: QtWidgets.QLabel
    label_14: QtWidgets.QLabel
    label_15: QtWidgets.QLabel
    label_16: QtWidgets.QLabel
    label_17: QtWidgets.QLabel
    label_18: QtWidgets.QLabel
    label_19: QtWidgets.QLabel
    label_2: QtWidgets.QLabel
    label_3: QtWidgets.QLabel
    label_4: QtWidgets.QLabel
    label_5: QtWidgets.QLabel
    label_6: QtWidgets.QLabel
    label_7: QtWidgets.QLabel
    label_8: QtWidgets.QLabel
    label_9: QtWidgets.QLabel
    lblNeedNormalize: QtWidgets.QLabel
    mainMenuBar: QtWidgets.QMenuBar
    mainTab: QtWidgets.QTabWidget
    mainToolBar: QtWidgets.QToolBar
    menuRun: QtWidgets.QMenu
    menu_Edit: QtWidgets.QMenu
    menu_File: QtWidgets.QMenu
    menu_Help: QtWidgets.QMenu
    pieContainer: QtWidgets.QFrame
    spinAngle: QtWidgets.QDoubleSpinBox
    spinElectricField: QtWidgets.QDoubleSpinBox
    spinFinalEnergy: QtWidgets.QDoubleSpinBox
    spinGasPressure: QtWidgets.QDoubleSpinBox
    spinGasTemperature: QtWidgets.QDoubleSpinBox
    spinMagneticField: QtWidgets.QDoubleSpinBox
    spinRealInteractions: QtWidgets.QSpinBox
    splitterGasRight: QtWidgets.QWidget
    splitterGases: QtWidgets.QSplitter
    splitterGasesLeft: QtWidgets.QWidget
    tabConfiguration: QtWidgets.QWidget
    tabExecution: QtWidgets.QWidget
    verticalLayout_2: QtWidgets.QVBoxLayout
    verticalLayout_4: QtWidgets.QVBoxLayout
    verticalLayout_5: QtWidgets.QVBoxLayout
    widget: QtWidgets.QWidget
    widget_2: QtWidgets.QWidget
    widget_3: QtWidgets.QWidget
    def setupUi(self, widget: QtWidgets.QMainWindow) -> None: ...
    def retranslateUi(self, widget: QtWidgets.QMainWindow) -> None: ...


from __future__ import annotations
from typing import TYPE_CHECKING

from PyQt6.QtCore import QProcess

if TYPE_CHECKING:
    from magboltz_gui.window.main_window import MagboltzGUI


class ProcessManager:

    def __init__(self, main_window: MagboltzGUI):
        self.main_window = main_window

    def run(self) -> None:

        self.process = QProcess(self.main_window)

        # Connect QProcess signals
        self.process.readyReadStandardOutput.connect(self.handle_stdout)
        self.process.readyReadStandardError.connect(self.handle_stderr)
        self.process.finished.connect(self.process_finished)

        self.run_process()

    def run_process(self) -> None:

        if self.main_window._currentInputFile is None:
            self.main_window.show_error("Error", "You have to open an input file first")
            return

        cmd = str(self.main_window.magboltzPath) if self.main_window.magboltzPath is not None else "magboltz"
        self.process.start(cmd)
        self.main_window.consoleOutput.clear()
        self.main_window.consoleOutput.append(
            f"<span style='color:blue;'>{cmd} &lt; {self.main_window._currentInputFile}</span>"
        )
        self.main_window.consoleOutput.append(f"")
        self.main_window.consoleOutput.append("Starting process...")

        with self.main_window._currentInputFile.open("r") as f:
            for line in f.readlines():
                self.process.write(f"{line}\n".encode("utf-8"))

    def handle_stdout(self) -> None:
        data = self.process.readAllStandardOutput().data()
        text = data.decode("utf-8")
        self.main_window.consoleOutput.append(text)

    def handle_stderr(self) -> None:
        data = self.process.readAllStandardError().data()
        text = data.decode("utf-8")
        self.main_window.consoleOutput.append(f"<span style='color:red;'>{text}</span>")

    def process_finished(self) -> None:
        self.main_window.consoleOutput.append("")
        self.main_window.consoleOutput.append("Process finished.")
        self.main_window.processes.remove(self)



from typing import Any
from PyQt6 import QtCore, QtGui, QtWidgets

class Ui_SearchGasForm:
    SearchGasForm: QtWidgets.QWidget
    dialogButtons: QtWidgets.QDialogButtonBox
    horizontalLayout: QtWidgets.QHBoxLayout
    horizontalLayout_2: QtWidgets.QHBoxLayout
    horizontalLayout_3: QtWidgets.QHBoxLayout
    horizontalSpacer: QtWidgets.QSpacerItem
    horizontalSpacer_2: QtWidgets.QSpacerItem
    label: QtWidgets.QLabel
    label_5: QtWidgets.QLabel
    label_note: QtWidgets.QLabel
    label_rating: QtWidgets.QLabel
    label_year: QtWidgets.QLabel
    searchBarText: QtWidgets.QLineEdit
    searchResultTable: QtWidgets.QTableWidget
    verticalLayout: QtWidgets.QVBoxLayout
    widget: QtWidgets.QWidget
    widget_2: QtWidgets.QWidget
    widget_3: QtWidgets.QWidget
    def setupUi(self, widget: QtWidgets.QWidget) -> None: ...
    def retranslateUi(self, widget: QtWidgets.QWidget) -> None: ...


#!/usr/bin/env python3
"""
ui2stub.py — Generate a .pyi typing stub from a Qt Designer .ui file (Qt6 / PyQt6)

Usage:
    python ui2stub.py path/to/mainwindow.ui
    # -> writes path/to/mainwindow.pyi (or use -o to change)

Place the .pyi next to the generated ui_*.py, or keep stubs in a typings/ dir
and add `mypy_path = ["typings"]` in your mypy config.
"""
from __future__ import annotations
import argparse
import xml.etree.ElementTree as ET
from pathlib import Path
from typing import Iterable, List, Tuple, Callable

# Module names for PyQt6. Adjust to PySide6 by changing imports in emit_pyi().
WIDGETS_MOD = "QtWidgets"
GUI_MOD = "QtGui"
CORE_MOD = "QtCore"

# Fallbacks
WIDGET_FALLBACK = f"{WIDGETS_MOD}.QWidget"
LAYOUT_FALLBACK = f"{WIDGETS_MOD}.QLayout"
ACTION_TYPE = f"{GUI_MOD}.QAction"
SPACER_TYPE = f"{WIDGETS_MOD}.QSpacerItem"

# Heuristics: most Qt6 widgets live in QtWidgets; QAction in QtGui
WIDGET_PREFIXES: Tuple[str, ...] = (
    "QWidget",
    "QMainWindow",
    "QDialog",
    "QFrame",
    "QGroupBox",
    "QPushButton",
    "QToolButton",
    "QRadioButton",
    "QCheckBox",
    "QComboBox",
    "QLineEdit",
    "QTextEdit",
    "QPlainTextEdit",
    "QLabel",
    "QListView",
    "QListWidget",
    "QTreeView",
    "QTreeWidget",
    "QTableView",
    "QTableWidget",
    "QTabWidget",
    "QStackedWidget",
    "QScrollArea",
    "QProgressBar",
    "QSlider",
    "QSpinBox",
    "QDoubleSpinBox",
    "QDateEdit",
    "QTimeEdit",
    "QDateTimeEdit",
    "QCalendarWidget",
    "QMenuBar",
    "QMenu",
    "QStatusBar",
    "QToolBar",
    "QDockWidget",
    "QGraphicsView",
)
LAYOUT_PREFIXES: Tuple[str, ...] = ("QVBoxLayout", "QHBoxLayout", "QGridLayout", "QFormLayout")


def qt_qualname(qt_class: str) -> str:
    """Map a Qt class name to a qualified PyQt6 name."""
    if qt_class == "Line":
        return "QtWidgets.QFrame"  # ← special case for Designer's "Line"
    if qt_class.startswith(LAYOUT_PREFIXES):
        return f"{WIDGETS_MOD}.{qt_class}"
    if qt_class.startswith(WIDGET_PREFIXES):
        return f"{WIDGETS_MOD}.{qt_class}"
    if qt_class == "QAction":
        return ACTION_TYPE
    # Best effort default (most widgets are in QtWidgets for PyQt6)
    return f"{WIDGETS_MOD}.{qt_class}"


def parse_ui(ui_path: Path) -> tuple[str, str, list[tuple[str, str]]]:
    """
    Return (ui_class_name, top_widget_qualified_type, attributes)
    attributes: list[(name, qualified_type)]
    """
    tree = ET.parse(ui_path)
    root = tree.getroot()
    ns_strip: Callable[[str], str] = lambda tag: tag.split("}")[-1]  # strip XML namespace if present

    # Ui class name: Ui_{<class>} (matches pyuic)
    form_name = root.findtext("class") or "Form"
    ui_class = f"Ui_{form_name}"

    # Top-level widget (QMainWindow/QDialog/QWidget…) -> setupUi signature
    top_widget = root.find("widget")
    top_qt_class = top_widget.get("class") if top_widget is not None else "QWidget"
    assert top_qt_class is not None
    top_param_type = qt_qualname(top_qt_class)

    attrs: List[Tuple[str, str]] = []

    # Traverse all elements and collect named things we want to expose
    for elem in root.iter():
        tag = ns_strip(elem.tag)

        if tag == "widget":
            name = elem.get("name")
            qt_class = elem.get("class")
            if name and qt_class:
                attrs.append((name, qt_qualname(qt_class)))

        elif tag == "layout":
            name = elem.get("name")
            qt_class = elem.get("class", "QLayout")
            if name:
                q = LAYOUT_FALLBACK if qt_class == "QLayout" else qt_qualname(qt_class)
                attrs.append((name, q))

        elif tag == "action":
            name = elem.get("name")
            if name:
                attrs.append((name, ACTION_TYPE))

        elif tag == "spacer":
            name = elem.get("name")
            if name:
                attrs.append((name, SPACER_TYPE))

    # De-duplicate by name, keep first occurrence
    seen = set()
    dedup: List[Tuple[str, str]] = []
    for name, typ in attrs:
        if name not in seen:
            seen.add(name)
            dedup.append((name, typ))

    return ui_class, top_param_type, dedup


def emit_pyi(ui_class: str, top_param_type: str, attrs: Iterable[Tuple[str, str]]) -> str:
    """Emit .pyi text for Ui_* class."""
    lines: List[str] = []
    lines.append("from typing import Any")
    lines.append("from PyQt6 import QtCore, QtGui, QtWidgets")
    lines.append("")
    lines.append(f"class {ui_class}:")
    if not attrs:
        lines.append("    def setupUi(self, widget: " + top_param_type + ") -> None: ...")
        lines.append("    def retranslateUi(self, widget: " + top_param_type + ") -> None: ...")
        return "\n".join(lines)

    # Sort attributes alphabetically for stable diffs
    for name, typ in sorted(attrs, key=lambda x: x[0]):
        lines.append(f"    {name}: {typ}")

    lines.append(f"    def setupUi(self, widget: {top_param_type}) -> None: ...")
    lines.append(f"    def retranslateUi(self, widget: {top_param_type}) -> None: ...")
    return "\n".join(lines)


def main() -> None:
    ap = argparse.ArgumentParser(description="Generate a .pyi stub from a Qt .ui file (PyQt6).")
    ap.add_argument("ui", type=Path, help="Path to .ui file")
    ap.add_argument("-o", "--out", type=Path, help="Output .pyi path (default: next to .ui)")
    args = ap.parse_args()

    ui_class, top_param_type, attrs = parse_ui(args.ui)
    text = emit_pyi(ui_class, top_param_type, attrs)

    out = args.out or args.ui.with_suffix(".pyi")
    out.write_text(text, encoding="utf-8")
    print(f"[ui2stub] Wrote {out}")


if __name__ == "__main__":
    main()



import signal
from importlib.resources import files

import sys
from typing import Any

from magboltz_gui.util.platform import ensure_qt_runtime_or_explain


def main() -> None:

    from PyQt6.QtGui import QIcon
    from PyQt6.QtWidgets import QApplication
    from PyQt6.QtCore import Qt
    from PyQt6.QtCore import QTimer
    from magboltz_gui.window.main_window import MagboltzGUI

    app: QApplication = QApplication(sys.argv)
    app.setApplicationName("Magboltz GUI")
    app.setOrganizationName("CERN")
    # IMPORTANT for Wayland/GNOME: this sets the app-id from a desktop file name
    app.setDesktopFileName(str(files("magboltz_gui.icons").joinpath("magboltz-gui.desktop")))
    app.setWindowIcon(QIcon(str(files("magboltz_gui.icons").joinpath("icon-192x192.png"))))

    signal.signal(signal.SIGINT, _sigint_handler)

    _keep_alive = QTimer()
    _keep_alive.timeout.connect(lambda: None)
    _keep_alive.start(200)

    window: MagboltzGUI = MagboltzGUI()
    window.setWindowFlag(Qt.WindowType.Window)
    window.show()
    sys.exit(app.exec())


def _sigint_handler(*_: Any) -> None:
    from PyQt6.QtWidgets import QApplication

    QApplication.quit()
    sys.exit(130)


if __name__ == "__main__":

    # Checking if we have QT available
    ensure_qt_runtime_or_explain()

    main()



# Form implementation generated from reading ui file 'src/magboltz_gui/ui/search.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_SearchGasForm(object):
    def setupUi(self, SearchGasForm):
        SearchGasForm.setObjectName("SearchGasForm")
        SearchGasForm.resize(377, 886)
        self.verticalLayout = QtWidgets.QVBoxLayout(SearchGasForm)
        self.verticalLayout.setObjectName("verticalLayout")
        self.searchBarText = QtWidgets.QLineEdit(parent=SearchGasForm)
        self.searchBarText.setObjectName("searchBarText")
        self.verticalLayout.addWidget(self.searchBarText)
        self.searchResultTable = QtWidgets.QTableWidget(parent=SearchGasForm)
        self.searchResultTable.setObjectName("searchResultTable")
        self.searchResultTable.setColumnCount(0)
        self.searchResultTable.setRowCount(0)
        self.verticalLayout.addWidget(self.searchResultTable)
        self.widget_2 = QtWidgets.QWidget(parent=SearchGasForm)
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(parent=self.widget_2)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.label_year = QtWidgets.QLabel(parent=self.widget_2)
        self.label_year.setObjectName("label_year")
        self.horizontalLayout_2.addWidget(self.label_year)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.label_5 = QtWidgets.QLabel(parent=self.widget_2)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_2.addWidget(self.label_5)
        self.label_rating = QtWidgets.QLabel(parent=self.widget_2)
        self.label_rating.setObjectName("label_rating")
        self.horizontalLayout_2.addWidget(self.label_rating)
        self.verticalLayout.addWidget(self.widget_2)
        self.widget_3 = QtWidgets.QWidget(parent=SearchGasForm)
        self.widget_3.setObjectName("widget_3")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.widget_3)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_note = QtWidgets.QLabel(parent=self.widget_3)
        self.label_note.setWordWrap(True)
        self.label_note.setObjectName("label_note")
        self.horizontalLayout_3.addWidget(self.label_note)
        self.verticalLayout.addWidget(self.widget_3)
        self.widget = QtWidgets.QWidget(parent=SearchGasForm)
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(977, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.dialogButtons = QtWidgets.QDialogButtonBox(parent=self.widget)
        self.dialogButtons.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.dialogButtons.setObjectName("dialogButtons")
        self.horizontalLayout.addWidget(self.dialogButtons)
        self.verticalLayout.addWidget(self.widget)

        self.retranslateUi(SearchGasForm)
        QtCore.QMetaObject.connectSlotsByName(SearchGasForm)

    def retranslateUi(self, SearchGasForm):
        _translate = QtCore.QCoreApplication.translate
        SearchGasForm.setWindowTitle(_translate("SearchGasForm", "Form"))
        self.searchBarText.setPlaceholderText(_translate("SearchGasForm", "(search here)"))
        self.label.setText(_translate("SearchGasForm", "Year: "))
        self.label_year.setText(_translate("SearchGasForm", "(year)"))
        self.label_5.setText(_translate("SearchGasForm", "Rating:"))
        self.label_rating.setText(_translate("SearchGasForm", "(rating)"))
        self.label_note.setText(_translate("SearchGasForm", "(note)"))



# Form implementation generated from reading ui file 'src/magboltz_gui/ui/preferences.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(788, 466)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(parent=Form)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(parent=self.groupBox)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.lineEdit = QtWidgets.QLineEdit(parent=self.groupBox)
        self.lineEdit.setObjectName("lineEdit")
        self.gridLayout.addWidget(self.lineEdit, 0, 1, 1, 1)
        self.toolButton = QtWidgets.QToolButton(parent=self.groupBox)
        self.toolButton.setObjectName("toolButton")
        self.gridLayout.addWidget(self.toolButton, 0, 2, 1, 1)
        self.verticalLayout.addWidget(self.groupBox)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=Form)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.groupBox.setTitle(_translate("Form", "General"))
        self.label.setText(_translate("Form", "Magboltz path"))
        self.toolButton.setText(_translate("Form", "..."))



from importlib.resources import files
from typing import Optional

from PyQt6 import uic
from PyQt6.QtCore import Qt
from PyQt6.QtWidgets import QDialog, QTableWidgetItem, QHeaderView, QTableWidget, QMessageBox, QWidget

from magboltz_gui.data.database import GasDatabase
from magboltz_gui.generated.ui_search import Ui_SearchGasForm


class SearchWindow(QDialog, Ui_SearchGasForm):

    def __init__(self, database: GasDatabase, parent: Optional[QWidget] = None) -> None:

        super().__init__(parent)
        self.database = database

        self.setupUi(self)
        # uic.loadUi(files("magboltz_gui.ui").joinpath("search.ui"), self) <- Not needed anymore because we use pyuic6

        self._selected_gas_id: int = 0

        self.dialogButtons.accepted.connect(self.onAccept)
        self.dialogButtons.rejected.connect(self.reject)

        self.searchResultTable.cellDoubleClicked.connect(self.onAccept)
        self.searchResultTable.itemSelectionChanged.connect(self.onSelectedRowChanged)
        self.searchBarText.textChanged.connect(self.refresh)

        self.refresh(self.searchBarText.text())
        self.onSelectedRowChanged()

    def onSelectedRowChanged(self) -> None:

        row = self.searchResultTable.currentRow()
        if row != -1:
            gas_id = int(self.searchResultTable.item(row, 0).text())
            gas = self.database.get(gas_id)
            self.label_year.setText(str(gas.year) if gas.year is not None else "-")
            self.label_rating.setText(f"{'★' * gas.rating}{'☆' * (5 - gas.rating)}" if gas.rating is not None else "-")
            self.label_note.setText(gas.note if gas.note is not None else "")
        else:
            self.label_year.setText("")
            self.label_rating.setText("")
            self.label_note.setText("")

    def onAccept(self) -> None:

        row = self.searchResultTable.currentRow()

        if row != -1:
            id_item = self.searchResultTable.item(row, 0)
            if id_item:
                self._selected_gas_id = int(id_item.text())
                self.accept()
                return

        QMessageBox.warning(self, "No Selection", "Please select a row first.")  # Parent  # Title  # Message

    def refresh(self, search: str) -> None:

        self.searchResultTable.clear()

        self.searchResultTable.setColumnCount(3)

        self.searchResultTable.setHorizontalHeaderLabels(["ID", "Name", "Formula"])

        # Make "Gas name" stretch to fill available space
        header = self.searchResultTable.horizontalHeader()
        header.setSectionResizeMode(0, QHeaderView.ResizeMode.ResizeToContents)
        header.setSectionResizeMode(1, QHeaderView.ResizeMode.Stretch)
        header.setSectionResizeMode(2, QHeaderView.ResizeMode.ResizeToContents)

        self.searchResultTable.verticalHeader().setVisible(False)
        self.searchResultTable.setSelectionBehavior(QTableWidget.SelectionBehavior.SelectRows)
        self.searchResultTable.setSelectionMode(QTableWidget.SelectionMode.SingleSelection)
        self.searchResultTable.setShowGrid(False)

        gas_filtered = []

        s = search.strip().lower()

        for gas in self.database.content:
            # This is the most inefficient search algorithm possible
            if (
                str(gas.id) == s
                or s in str(gas.name.lower())
                or (gas.formula is not None and s in gas.formula.lower())
                or (gas.year is not None and str(gas.year) == s)
                or (gas.note is not None and s in gas.note.lower())
            ):
                gas_filtered.append(gas)

        self.searchResultTable.setRowCount(len(gas_filtered))

        i = 0
        for gas in gas_filtered:

            widget_id = QTableWidgetItem(str(gas.id))
            widget_id.setTextAlignment(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)

            widget_name = QTableWidgetItem(gas.name)

            widget_formula = QTableWidgetItem(gas.formula or "")

            widget_id.setFlags(widget_id.flags() & ~Qt.ItemFlag.ItemIsEditable)
            widget_name.setFlags(widget_name.flags() & ~Qt.ItemFlag.ItemIsEditable)
            widget_formula.setFlags(widget_formula.flags() & ~Qt.ItemFlag.ItemIsEditable)

            self.searchResultTable.setItem(i, 0, widget_id)
            self.searchResultTable.setItem(i, 1, widget_name)
            self.searchResultTable.setItem(i, 2, widget_formula)

            i += 1

    def setCurrentGasId(self, gas_id: int) -> None:
        self._selected_gas_id = gas_id

    def getSelectedGasId(self) -> int:
        return self._selected_gas_id

    def accept(self) -> None:
        super().accept()
